package com.oqpo.api.web.controller.pomng;

import com.oqpo.api.enums.DataMessageCode;
import com.oqpo.api.enums.SystemMessageCode;
import com.oqpo.api.service.oper.AfflService;
import com.oqpo.api.service.pomng.PchReqService;
import com.oqpo.api.web.dto.request.oper.affl.AfflGridRequest;
import com.oqpo.api.web.dto.request.oper.affl.AfflShopBaseInfoGridRequest;
import com.oqpo.api.web.dto.request.oper.affl.SaveAfflRequest;
import com.oqpo.api.web.dto.request.pomng.pchReq.PchReqSearchGridRequest;
import com.oqpo.api.web.dto.request.pomng.pchReq.SavePchReqlRequest;
import com.oqpo.api.web.dto.response.DataResponse;
import com.oqpo.api.web.dto.response.GridResponse;
import com.oqpo.api.web.dto.response.SaveResponse;
import com.oqpo.api.web.dto.response.oper.affl.AfflShopInfoResponse;
import com.oqpo.api.web.dto.response.pomng.pchReq.PchReqInfoResponse;
import com.oqpo.api.web.dto.response.pomng.pchReq.PchReqInitInfoResponse;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

import javax.validation.Valid;

@Slf4j
@RestController
@RequestMapping("/api/pomng")
@Api(tags = {"구매요청 관리 (PchReqController) -- 진행중"})
public class PchReqController {
    @Autowired
    private PchReqService pchReqService;


    /**
     * 설명 : 구매요청 초기 정보
     *
     * @param userId
     * @return
     * @throws Exception
     */
    @ApiImplicitParams({
            @ApiImplicitParam(name = "X-AUTH-TOKEN", value = "CONN-KEY", required = true, dataType = "String", paramType = "header")
    })
    @ApiOperation(value = "구매요청 초기 정보")
    @GetMapping("/init-pchReq")
    public ResponseEntity<PchReqInitInfoResponse> selectPchReqInitInfo(@ApiIgnore String userId) throws Exception {
        return ResponseEntity.ok(pchReqService.selectPchReqInitInfo(userId));
    }

    /**
     * 설명 : 구매요청 그리드 리스트
     *
     * @param afflShopBaseInfoGridRequest
     * @return
     * @throws Exception
     */

    @ApiImplicitParams({
            @ApiImplicitParam(name = "X-AUTH-TOKEN", value = "CONN-KEY", required = true, dataType = "String", paramType = "header")
    })
    @ApiOperation(value = " 구매요청 그리드 리스트")
    @PostMapping("/detail-grid-list")
    public ResponseEntity<GridResponse> detailGridList(@RequestBody @Valid PchReqSearchGridRequest pchReqSearchGridRequest) throws Exception {
        return ResponseEntity.ok(pchReqService.selectPchReqGridList(pchReqSearchGridRequest.getSBrandId(), pchReqSearchGridRequest.getFromDt(), pchReqSearchGridRequest.getToDt(), pchReqSearchGridRequest.getSWhsId(), pchReqSearchGridRequest.getSPchReqStCd(), pchReqSearchGridRequest.getSItemNm(),  pchReqSearchGridRequest.toDTO(pchReqSearchGridRequest)));
    }


    /**
     * 설명 : 구매요청 정보
     *
     * @param brandId, pchReqUnqNo
     * @return
     * @throws Exception
     */
    @ApiImplicitParams({
            @ApiImplicitParam(name = "X-AUTH-TOKEN", value = "CONN-KEY", required = true, dataType = "String", paramType = "header"),
            @ApiImplicitParam(name = "brandId", value = "가맹점번호", required = true, dataType = "String", paramType = "query"),
            @ApiImplicitParam(name = "pchReqUnqNo", value = "구매요청번호", required = true, dataType = "String", paramType = "query")
    })
    @ApiOperation(value = "구매요청 정보")
    @GetMapping("/info-pchReq")
    public ResponseEntity<PchReqInfoResponse> infoPchReq(@RequestParam(value = "brandId") String brandId, @RequestParam(value = "pchReqUnqNo") String pchReqUnqNo) throws Exception {
        return ResponseEntity.ok(pchReqService.selectPchReqInfo(brandId,pchReqUnqNo));
    }


    /**
     * 설명 : 구매요청 정보 저장
     *
     * @param userId
     * @param savePchReqlRequest
     * @return SaveResponse
     * @throws Exception
     */
    @ApiImplicitParams({
            @ApiImplicitParam(name = "X-AUTH-TOKEN", value = "CONN-KEY", required = true, dataType = "String", paramType = "header")
    })
    @ApiOperation(value = "구매요청 정보 저장")
    @PostMapping("/save-pchReq")
    public ResponseEntity<SaveResponse> savePchReq(@ApiIgnore String userId, @ApiIgnore String userNm, @RequestBody @Valid SavePchReqlRequest savePchReqlRequest) throws Exception {
        pchReqService.savePchReqInfo(userId, userNm,  savePchReqlRequest);
        return ResponseEntity.ok(SaveResponse.toDTO(SystemMessageCode.SAVE_OK));
    }


}
